@@ -1,7 +1,7 @@
 # Automated tests for the `coloredlogs' package.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: July 15, 2017
+# Last Change: January 17, 2018
 # URL: https://coloredlogs.readthedocs.io
 
 """Automated tests for the `coloredlogs` package."""
@@ -12,16 +12,16 @@ import imp
 import logging
 import logging.handlers
 import os
-import random
 import re
-import string
 import subprocess
 import sys
 import tempfile
-import unittest
 
 # External dependencies.
-from humanfriendly.terminal import ansi_wrap
+from humanfriendly.compat import StringIO
+from humanfriendly.terminal import ANSI_COLOR_CODES, ansi_style, ansi_wrap
+from humanfriendly.testing import PatchedItem, TestCase, retry
+from humanfriendly.text import format, random_string
 from mock import MagicMock
 
 # The module we're testing.
@@ -47,12 +47,16 @@ from coloredlogs import (
     walk_propagation_tree,
 )
 from coloredlogs.syslog import SystemLogging, match_syslog_handler
-from coloredlogs.converter import capture, convert
+from coloredlogs.converter import (
+    ColoredCronMailer,
+    EIGHT_COLOR_PALETTE,
+    capture,
+    convert,
+)
 
 # External test dependencies.
 from capturer import CaptureOutput
 from verboselogs import VerboseLogger
-from humanfriendly.compat import StringIO
 
 # Compiled regular expression that matches a single line of output produced by
 # the default log format (does not include matching of ANSI escape sequences).
@@ -78,9 +82,6 @@ PATTERN_INCLUDING_MILLISECONDS = re.compile(r'''
     \s (?P<message> .* )
 ''', re.VERBOSE)
 
-# The pathname of the system log file on Ubuntu Linux (my laptops and Travis CI).
-UNIX_SYSTEM_LOG = '/var/log/syslog'
-
 
 def setUpModule():
     """Speed up the tests by disabling the demo's artificial delay."""
@@ -88,16 +89,30 @@ def setUpModule():
     coloredlogs.demo.DEMO_DELAY = 0
 
 
-class ColoredLogsTestCase(unittest.TestCase):
+class ColoredLogsTestCase(TestCase):
 
     """Container for the `coloredlogs` tests."""
 
+    def find_system_log(self):
+        """Find the system log file or skip the current test."""
+        filename = ('/var/log/system.log' if sys.platform == 'darwin' else (
+            '/var/log/syslog' if 'linux' in sys.platform else None
+        ))
+        if not filename:
+            self.skipTest("Location of system log file unknown!")
+        elif not os.path.isfile(filename):
+            self.skipTest("System log file not found! (%s)" % filename)
+        elif not os.access(filename, os.R_OK):
+            self.skipTest("Insufficient permissions to read system log file! (%s)" % filename)
+        else:
+            return filename
+
     def test_level_to_number(self):
         """Make sure :func:`level_to_number()` works as intended."""
         # Make sure the default levels are translated as expected.
         assert level_to_number('debug') == logging.DEBUG
         assert level_to_number('info') == logging.INFO
-        assert level_to_number('warn') == logging.WARNING
+        assert level_to_number('warning') == logging.WARNING
         assert level_to_number('error') == logging.ERROR
         assert level_to_number('fatal') == logging.FATAL
         # Make sure bogus level names don't blow up.
@@ -170,36 +185,47 @@ class ColoredLogsTestCase(unittest.TestCase):
             assert not isinstance(handler.formatter, ColoredFormatter)
 
     def test_system_logging(self):
-        """Make sure the :mod:`coloredlogs.syslog` module works."""
+        """Make sure the :class:`coloredlogs.syslog.SystemLogging` context manager works."""
+        system_log_file = self.find_system_log()
         expected_message = random_string(50)
         with SystemLogging(programname='coloredlogs-test-suite') as syslog:
-            logging.info("%s", expected_message)
-            if syslog and os.path.isfile('/var/log/syslog'):
-                with open('/var/log/syslog') as handle:
-                    assert any(expected_message in line for line in handle)
+            if not syslog:
+                return self.skipTest("couldn't connect to syslog daemon")
+            # When I tried out the system logging support on macOS 10.13.1 on
+            # 2018-01-05 I found that while WARNING and ERROR messages show up
+            # in the system log DEBUG and INFO messages don't. This explains
+            # the importance of the level of the log message below.
+            logging.error("%s", expected_message)
+        # Retry the following assertion (for up to 60 seconds) to give the
+        # logging daemon time to write our log message to disk. This
+        # appears to be needed on MacOS workers on Travis CI, see:
+        # https://travis-ci.org/xolox/python-coloredlogs/jobs/325245853
+        retry(lambda: check_contents(system_log_file, expected_message, True))
 
     def test_syslog_shortcut_simple(self):
         """Make sure that ``coloredlogs.install(syslog=True)`` works."""
+        system_log_file = self.find_system_log()
+        expected_message = random_string(50)
         with cleanup_handlers():
-            expected_message = random_string(50)
+            # See test_system_logging() for the importance of this log level.
             coloredlogs.install(syslog=True)
-            logging.info("%s", expected_message)
-            if os.path.isfile(UNIX_SYSTEM_LOG):
-                with open(UNIX_SYSTEM_LOG) as handle:
-                    assert any(expected_message in line for line in handle)
+            logging.error("%s", expected_message)
+        # See the comments in test_system_logging() on why this is retried.
+        retry(lambda: check_contents(system_log_file, expected_message, True))
 
     def test_syslog_shortcut_enhanced(self):
-        """Make sure that ``coloredlogs.install(syslog='warn')`` works."""
+        """Make sure that ``coloredlogs.install(syslog='warning')`` works."""
+        system_log_file = self.find_system_log()
+        the_expected_message = random_string(50)
+        not_an_expected_message = random_string(50)
         with cleanup_handlers():
-            the_expected_message = random_string(50)
-            not_an_expected_message = random_string(50)
-            coloredlogs.install(syslog='warn')
-            logging.info("%s", not_an_expected_message)
-            logging.warning("%s", the_expected_message)
-            if os.path.isfile(UNIX_SYSTEM_LOG):
-                with open(UNIX_SYSTEM_LOG) as handle:
-                    assert any(the_expected_message in line for line in handle)
-                    assert not any(not_an_expected_message in line for line in handle)
+            # See test_system_logging() for the importance of these log levels.
+            coloredlogs.install(syslog='error')
+            logging.warning("%s", not_an_expected_message)
+            logging.error("%s", the_expected_message)
+        # See the comments in test_system_logging() on why this is retried.
+        retry(lambda: check_contents(system_log_file, the_expected_message, True))
+        retry(lambda: check_contents(system_log_file, not_an_expected_message, False))
 
     def test_name_normalization(self):
         """Make sure :class:`~coloredlogs.NameNormalizer` works as intended."""
@@ -262,7 +288,10 @@ class ColoredLogsTestCase(unittest.TestCase):
         # NOTICE -> WARNING.
         decrease_verbosity()
         assert get_level() == logging.WARNING
-        # WARNING -> ERROR.
+        # WARNING -> SUCCESS.
+        decrease_verbosity()
+        assert get_level() == logging.SUCCESS
+        # SUCCESS -> ERROR.
         decrease_verbosity()
         assert get_level() == logging.ERROR
         # ERROR -> CRITICAL.
@@ -347,7 +376,7 @@ class ColoredLogsTestCase(unittest.TestCase):
         for method, severity in ((logger.debug, 'DEBUG'),
                                  (logger.info, 'INFO'),
                                  (logger.verbose, 'VERBOSE'),
-                                 (logger.warning, 'WARN'),
+                                 (logger.warning, 'WARNING'),
                                  (logger.error, 'ERROR'),
                                  (logger.critical, 'CRITICAL')):
             # Prepare the text.
@@ -364,13 +393,63 @@ class ColoredLogsTestCase(unittest.TestCase):
 
     def test_html_conversion(self):
         """Check the conversion from ANSI escape sequences to HTML."""
-        ansi_encoded_text = 'I like %s - www.eelstheband.com' % ansi_wrap('birds', bold=True, color='blue')
-        assert ansi_encoded_text == 'I like \x1b[1;34mbirds\x1b[0m - www.eelstheband.com'
-        html_encoded_text = convert(ansi_encoded_text)
-        assert html_encoded_text == (
-            '<code>I like <span style="font-weight:bold;color:blue">birds</span> - '
-            '<a href="http://www.eelstheband.com" style="color:inherit">www.eelstheband.com</a></code>'
+        # Check conversion of colored text.
+        for color_name, ansi_code in ANSI_COLOR_CODES.items():
+            ansi_encoded_text = 'plain text followed by %s text' % ansi_wrap(color_name, color=color_name)
+            expected_html = format(
+                '<code>plain text followed by <span style="color:{css}">{name}</span> text</code>',
+                css=EIGHT_COLOR_PALETTE[ansi_code], name=color_name,
+            )
+            self.assertEquals(expected_html, convert(ansi_encoded_text))
+        # Check conversion of bright colored text.
+        expected_html = '<code><span style="color:#FF0">bright yellow</span></code>'
+        self.assertEquals(expected_html, convert(ansi_wrap('bright yellow', color='yellow', bright=True)))
+        # Check conversion of text with a background color.
+        expected_html = '<code><span style="background-color:#DE382B">red background</span></code>'
+        self.assertEquals(expected_html, convert(ansi_wrap('red background', background='red')))
+        # Check conversion of text with a bright background color.
+        expected_html = '<code><span style="background-color:#F00">bright red background</span></code>'
+        self.assertEquals(expected_html, convert(ansi_wrap('bright red background', background='red', bright=True)))
+        # Check conversion of text that uses the 256 color mode palette as a foreground color.
+        expected_html = '<code><span style="color:#FFAF00">256 color mode foreground</span></code>'
+        self.assertEquals(expected_html, convert(ansi_wrap('256 color mode foreground', color=214)))
+        # Check conversion of text that uses the 256 color mode palette as a background color.
+        expected_html = '<code><span style="background-color:#AF0000">256 color mode background</span></code>'
+        self.assertEquals(expected_html, convert(ansi_wrap('256 color mode background', background=124)))
+        # Check that invalid 256 color mode indexes don't raise exceptions.
+        expected_html = '<code>plain text expected</code>'
+        self.assertEquals(expected_html, convert('\x1b[38;5;256mplain text expected\x1b[0m'))
+        # Check conversion of bold text.
+        expected_html = '<code><span style="font-weight:bold">bold text</span></code>'
+        self.assertEquals(expected_html, convert(ansi_wrap('bold text', bold=True)))
+        # Check conversion of underlined text.
+        expected_html = '<code><span style="text-decoration:underline">underlined text</span></code>'
+        self.assertEquals(expected_html, convert(ansi_wrap('underlined text', underline=True)))
+        # Check conversion of strike-through text.
+        expected_html = '<code><span style="text-decoration:line-through">strike-through text</span></code>'
+        self.assertEquals(expected_html, convert(ansi_wrap('strike-through text', strike_through=True)))
+        # Check conversion of inverse text.
+        expected_html = '<code><span style="background-color:#FFC706;color:#000">inverse</span></code>'
+        self.assertEquals(expected_html, convert(ansi_wrap('inverse', color='yellow', inverse=True)))
+        # Check conversion of URLs.
+        for sample_text in 'www.python.org', 'http://coloredlogs.rtfd.org', 'https://coloredlogs.rtfd.org':
+            sample_url = sample_text if '://' in sample_text else ('http://' + sample_text)
+            expected_html = '<code><a href="%s" style="color:inherit">%s</a></code>' % (sample_url, sample_text)
+            self.assertEquals(expected_html, convert(sample_text))
+        # Check that the capture pattern for URLs doesn't match ANSI escape
+        # sequences and also check that the short hand for the 0 reset code is
+        # supported. These are tests for regressions of bugs found in
+        # coloredlogs <= 8.0.
+        reset_short_hand = '\x1b[0m'
+        blue_underlined = ansi_style(color='blue', underline=True)
+        ansi_encoded_text = '<%shttps://coloredlogs.readthedocs.io%s>' % (blue_underlined, reset_short_hand)
+        expected_html = (
+            '<code>&lt;<span style="color:#006FB8;text-decoration:underline">'
+            '<a href="https://coloredlogs.readthedocs.io" style="color:inherit">'
+            'https://coloredlogs.readthedocs.io'
+            '</a></span>&gt;</code>'
         )
+        self.assertEquals(expected_html, convert(ansi_encoded_text))
 
     def test_output_interception(self):
         """Test capturing of output from external commands."""
@@ -378,6 +457,18 @@ class ColoredLogsTestCase(unittest.TestCase):
         actual_output = capture(['echo', expected_output])
         assert actual_output.strip() == expected_output.strip()
 
+    def test_enable_colored_cron_mailer(self):
+        """Test that automatic ANSI to HTML conversion when running under ``cron`` can be enabled."""
+        with PatchedItem(os.environ, 'CONTENT_TYPE', 'text/html'):
+            with ColoredCronMailer() as mailer:
+                assert mailer.is_enabled
+
+    def test_disable_colored_cron_mailer(self):
+        """Test that automatic ANSI to HTML conversion when running under ``cron`` can be disabled."""
+        with PatchedItem(os.environ, 'CONTENT_TYPE', 'text/plain'):
+            with ColoredCronMailer() as mailer:
+                assert not mailer.is_enabled
+
     def test_auto_install(self):
         """Test :func:`coloredlogs.auto_install()`."""
         needle = random_string()
@@ -411,6 +502,19 @@ class ColoredLogsTestCase(unittest.TestCase):
         # Make sure the output is encoded as HTML.
         assert '<span' in output
 
+    def test_empty_conversion(self):
+        """
+        Test that conversion of empty output produces no HTML.
+
+        This test was added because I found that ``coloredlogs --convert`` when
+        used in a cron job could cause cron to send out what appeared to be
+        empty emails. On more careful inspection the body of those emails was
+        ``<code></code>``. By not emitting the wrapper element when no other
+        HTML is generated, cron will not send out an email.
+        """
+        output = main('coloredlogs', '--convert', 'true', capture=True)
+        assert not output.strip()
+
     def test_implicit_usage_message(self):
         """Test that the usage message is shown when no actions are given."""
         assert 'Usage:' in main('coloredlogs', capture=True)
@@ -420,6 +524,12 @@ class ColoredLogsTestCase(unittest.TestCase):
         assert 'Usage:' in main('coloredlogs', '--help', capture=True)
 
 
+def check_contents(filename, contents, match):
+    """Check if a line in a file contains an expected string."""
+    with open(filename) as handle:
+        assert any(contents in line for line in handle) == match
+
+
 def main(*arguments, **options):
     """Wrap the command line interface to make it easier to test."""
     capture = options.get('capture', False)
@@ -437,11 +547,6 @@ def main(*arguments, **options):
         sys.stdout = saved_stdout
 
 
-def random_string(length=25):
-    """Generate a random string."""
-    return ''.join(random.choice(string.ascii_letters) for i in range(25))
-
-
 @contextlib.contextmanager
 def mocked_colorama_module(init_function):
     """Context manager to ease testing of colorama integration."""
